@@ -58,6 +58,21 @@ sub get_attributes {
     return \@attributes;
 }
 
+# accessor code for normal objects
+# (overloaded in D::O::Singleton for instance)
+sub _setter_code {
+    my ($class, $attr) = @_;
+    sub {
+        my ($self, $value) = @_;
+        if (@_ == 1) {
+            return $self->{$attr};
+        }
+        else {
+            return $self->{$attr} = $value;
+        }
+    };
+}
+
 # accessors builder
 sub attributes {
     my ($class, @attributes) = @_;
@@ -67,20 +82,19 @@ sub attributes {
 
     # define setters and getters for each attribute
     foreach my $attr (@attributes) {
-        my $code = sub {
-            my ($self, $value) = @_;
-            if (@_ == 1) {
-                return $self->{$attr};
-            }
-            else {
-                return $self->{$attr} = $value;
-            }
-        };
+        my $code = $class->_setter_code($attr);
         my $method = "${class}::${attr}";
         { no strict 'refs'; *$method = $code; }
     }
 }
 
+sub attributes_defaults {
+    my ($self, %defaults) = @_;
+    while (my ($k, $v) = each %defaults) {
+        exists $self->{$k} or $self->{$k} = $v;
+    }
+}
+
 1;
 
 __END__
@@ -105,7 +119,7 @@ Dancer::Object - Objects base class for Dancer
 
 =head1 DESCRIPTION
 
-While we B<love> L<Moose>, we can't use it for Dancer and stlil keep Dancer
+While we B<love> L<Moose>, we can't use it for Dancer and still keep Dancer
 minimal, so we wrote Dancer::Object instead.
 
 It provides you with attributes and an initializer.
@@ -141,6 +155,30 @@ Generates attributes for whatever object is extending Dancer::Object and saves
 them in an internal hashref so they can be later fetched using
 C<get_attributes>.
 
+For each defined attribute you can access its value using:
+
+  $self->your_attribute_name;
+
+To set a value use
+
+  $self->your_attribute_name($value);
+
+Nevertheless, you can continue to use these attributes as hash keys,
+as usual with blessed hash references:
+
+  $self->{your_attribute_name} = $value;
+
+Although this is possible we defend you should use the method
+approach, as it maintains compatibility in case C<Dancer::Object>
+structure changes in the future.
+
+=head2 attributes_defaults
+
+  $self->attributes_defaults(length => 2);
+
+given a hash (not a hashref), makes sure an object has the given attributes
+default values. Usually called from within an C<init> function.
+
 =head1 AUTHOR
 
 Alexis Sukrieh
