@@ -13,7 +13,7 @@ use Dancer::Request;
 use Dancer::Response;
 use Dancer::Serializer;
 use Dancer::Config 'setting';
-use Dancer::FileUtils qw(path dirname read_file_content open_file);
+use Dancer::FileUtils qw(path path_or_empty dirname read_file_content open_file);
 use Dancer::SharedData;
 use Dancer::Logger;
 use Dancer::MIME;
@@ -54,8 +54,10 @@ sub render_error {
 sub response_with_headers {
     my $response = Dancer::SharedData->response();
 
-   $response->{headers} ||= HTTP::Headers->new;
-   $response->header('X-Powered-By' => "Perl Dancer ${Dancer::VERSION}");
+    $response->{headers} ||= HTTP::Headers->new;
+    my $powered_by = "Perl Dancer " . $Dancer::VERSION;
+    $response->header('X-Powered-By' => $powered_by);
+    $response->header('Server'       => $powered_by);
 
     return $response;
 }
@@ -145,10 +147,22 @@ sub serialize_response_if_needed {
 }
 
 sub get_file_response {
-    my $request     = Dancer::SharedData->request;
-    my $path_info   = $request->path_info;
-    my $app         = Dancer::App->current;
-    my $static_file = path($app->setting('public'), $path_info);
+    my $request   = Dancer::SharedData->request;
+    my $path_info = $request->path_info;
+
+    # requests that have \0 in path are forbidden
+    if ( $path_info =~ /\0/ ) {
+        _bad_request();
+        return 1;
+    }
+
+    my $app = Dancer::App->current;
+    # TODO: this should be later removed with a check whether the file exists
+    # and then returning a 404, path_or_empty should be removed
+    my $static_file = path_or_empty( $app->setting('public'), $path_info );
+
+    return if ( !$static_file
+        || index( $static_file, path( $app->setting('public') ) ) != 0 );
 
     return Dancer::Renderer->get_file_response_for_path( $static_file, undef,
         $request->content_type );
@@ -156,7 +170,6 @@ sub get_file_response {
 
 sub get_file_response_for_path {
     my ($class, $static_file, $status, $mime) = @_;
-    $status ||= 200;
 
     if ( -f $static_file ) {
         Dancer::Factory::Hook->execute_hooks( 'before_file_render',
@@ -165,7 +178,7 @@ sub get_file_response_for_path {
         my $fh = open_file( '<', $static_file );
         binmode $fh;
         my $response = Dancer::SharedData->response() || Dancer::Response->new();
-        $response->status($status);
+        $response->status($status) if ($status);
         $response->header('Content-Type' => (($mime && _get_full_mime_type($mime)) ||
                                              _get_mime_type($static_file)));
         $response->content($fh);
@@ -189,6 +202,12 @@ sub _get_mime_type {
     return $mime->for_file($file);
 }
 
+sub _bad_request{
+    my $response = Dancer::SharedData->response() || Dancer::Response->new();
+    $response->status(400);
+    $response->content('Bad Request');
+}
+
 # set of builtin templates needed by Dancer when rendering HTML pages
 sub templates {
     my $charset = setting('charset') || 'UTF-8';
