@@ -3,10 +3,12 @@ package Dancer::SharedData;
 use strict;
 use warnings;
 use Dancer::Timer;
+use Dancer::Response;
 
 # shared variables
-my $vars   = {};
+my $vars = {};
 sub vars {$vars}
+
 sub var {
     my ($class, $key, $value) = @_;
     $vars->{$key} = $value if (@_ == 3);
@@ -21,18 +23,30 @@ sub headers { (@_ == 2) ? $_headers = $_[1] : $_headers }
 my $_request;
 sub request { (@_ == 2) ? $_request = $_[1] : $_request }
 
+# current response
+my $_response;
+sub response {
+    if (@_ == 2) {
+        $_response = $_[1];
+    }else{
+        $_response = Dancer::Response->new() if !defined $_response;
+        return $_response;
+    }
+}
+sub reset_response { $_response = undef }
+
 # request timer
 my $_timer;
-sub timer { $_timer ||= Dancer::Timer->new };
-sub reset_timer { $_timer = Dancer::Timer->new };
+sub timer { $_timer ||= Dancer::Timer->new }
+sub reset_timer { $_timer = Dancer::Timer->new }
 
 # purging accessor
 sub reset_all {
-    $vars   = {};
+    $vars = {};
     undef $_request;
-    undef $_timer;
     undef $_headers;
+    reset_timer();
+    reset_response();
 }
 
-
 'Dancer::SharedData';
